@@ -49,7 +49,7 @@ class Consumer(object):
     def pause(self, seconds):
         logger.info('Pausing for %i seconds...', seconds)
         self._cancel()
-        self.queue.sleep(seconds)
+        self.queue.channel.connection.sleep(seconds)
         logger.info('Resuming')
         self._consume()
 
@@ -90,7 +90,7 @@ class Consumer(object):
 
     def _process_data_events(self):
         while not self._stop_processing_data_events.is_set():
-            self.queue.process_data_events()
+            self.queue.channel.connection.process_data_events()
 
 
 class MessageIterator(object):
