@@ -26,3 +26,8 @@ class Timeout(KuyrukError):
 class InvalidTask(KuyrukError):
     """Raised when the message from queue is not valid."""
     pass
+
+
+class InvalidCall(KuyrukError):
+    """Raised when a class task is called with invalid arguments."""
+    pass
