@@ -1,15 +1,14 @@
 from __future__ import absolute_import
 import os
-import sys
 import errno
 import signal
 import socket
 import string
 import logging
-import tempfile
+import resource
 import itertools
-import subprocess
 from time import time, sleep
+from collections import namedtuple
 
 from setproctitle import setproctitle
 
@@ -60,9 +59,13 @@ class Master(KuyrukProcess):
 
     def shutdown_workers(self, kill=False):
         """Sends shutdown signal to all workers."""
-        kill_signal = signal.SIGKILL if kill else signal.SIGTERM
+        if kill:
+            fn, sig = os.killpg, signal.SIGKILL
+        else:
+            fn, sig = os.kill, signal.SIGTERM
+
         for worker in self.workers:
-            os.kill(worker.pid, kill_signal)
+            fn(worker.pid, sig)
 
     def wait_for_workers(self):
         """Loops until any of the self.workers is alive.
@@ -96,7 +99,7 @@ class Master(KuyrukProcess):
         logger.debug(self.workers)
 
     def spawn_new_worker(self, queue):
-        worker = WorkerProcess(self.config, queue)
+        worker = WorkerProcess(self.kuyruk, queue)
         worker.start()
         self.workers.append(worker)
 
@@ -106,6 +109,10 @@ class Master(KuyrukProcess):
         signal.signal(signal.SIGQUIT, self.handle_sigquit)
         signal.signal(signal.SIGABRT, self.handle_sigabrt)
 
+        # Ignore SIGCHLD explicitly so the kernel to
+        # automatically reap child processes.
+        signal.signal(signal.SIGCHLD, signal.SIG_IGN)
+
     def handle_sigterm(self, signum, frame):
         logger.warning("Handling SIGTERM")
         self.warm_shutdown()
@@ -118,10 +125,9 @@ class Master(KuyrukProcess):
         logger.warning("Handling SIGABRT")
         self.abort()
 
-    def warm_shutdown(self, sigint=False):
-        super(Master, self).warm_shutdown(sigint)
-        if not sigint:
-            self.shutdown_workers()
+    def warm_shutdown(self):
+        super(Master, self).warm_shutdown()
+        self.shutdown_workers()
 
     def cold_shutdown(self):
         logger.warning("Cold shutdown")
@@ -148,35 +154,52 @@ class Master(KuyrukProcess):
 
 class WorkerProcess(object):
 
-    def __init__(self, config, queue):
-        self.config = config
+    def __init__(self, kuyruk, queue):
+        self.kuyruk = kuyruk
         self.queue = queue
-        self.popen = None
 
     def start(self):
-        """Runs the worker by starting another Python interpreter."""
-        # Pass configuration values to the worker
-        fd, path = tempfile.mkstemp()
-        self.config.export(fd)
+        pid = os.fork()
+        if pid:
+            # master
+            self.pid = pid
+        else:
+            # child
+            self.run_worker()
 
-        command = [
-            sys.executable, '-u', '-m', 'kuyruk.__main__',
-            '--config', path,
-            '--delete-config',
-            'worker',
-            '--queue', self.queue
-        ]
-        self.popen = subprocess.Popen(
-            command, stdout=sys.stdout, stderr=sys.stderr,
-            bufsize=1, close_fds=True)
+    def run_worker(self):
+        os.setpgrp()
+        self.close_fds()
+        Args = namedtuple('Args', 'queue')
+        args = Args(queue=self.queue)
+        import kuyruk.__main__
+        kuyruk.__main__.worker(self.kuyruk, args)
+        os._exit(0)
 
-    @property
-    def pid(self):
-        return self.popen.pid
+    def close_fds(self):
+        maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
+        if maxfd == resource.RLIM_INFINITY:
+            maxfd = 1024
+
+        # Iterate through and close all file descriptors.
+        # Do not close stdin, out and err (0, 1, 2)
+        for fd in range(3, maxfd):
+            try:
+                os.close(fd)
+            except OSError:  # fd wasn't open
+                pass
 
     def is_alive(self):
-        self.popen.poll()
-        return self.popen.returncode is None
+        """Send signal 0 to process to check if it is alive."""
+        logger.debug("Cheking if the worker is alive? pid=%s", self.pid)
+        try:
+            os.kill(self.pid, 0)
+        except OSError:
+            logger.debug("No")
+            return False
+        else:
+            logger.debug("Yes")
+            return True
 
     def kill_pg(self):
         """Kills the process with their children. Does not raise exception
